@@ -3,7 +3,7 @@
 
 package HiD::Role::IsPost;
 {
-  $HiD::Role::IsPost::VERSION = '0.4';
+  $HiD::Role::IsPost::VERSION = '0.5';
 }
 BEGIN {
   $HiD::Role::IsPost::AUTHORITY = 'cpan:GENEHACK';
@@ -94,11 +94,26 @@ has date => (
 );
 
 
-### TODO parse tags out of metadata
 has tags => (
   is      => 'ro' ,
   isa     => 'ArrayRef',
-  default => sub {[]} ,
+  default => sub {
+    my $self = shift;
+
+    if ( my $tag = $self->get_metadata( 'tag' )) {
+      return [ $tag ];
+    }
+    elsif ( my $tags = $self->get_metadata( 'tags' )) {
+      if ( ref $tags ) {
+        return [ @$tags ];
+      }
+      else {
+        my @tags = split /\s/ , $tags;
+        return [ @tags ];
+      }
+    }
+    else { return [] }
+  } ,
 );
 
 
@@ -180,7 +195,7 @@ DateTime object for this post.
 
 =head1 VERSION
 
-version 0.4
+version 0.5
 
 =head1 AUTHOR
 
